% Plots two versions of the class Mandlebrot Set representations of the
% Argand Diagram formed by iterations of the complex mapping z -> z^2 + z0
function mandelbrot
N = 500; %Vertical pixels (horizontal pixels is twice this)
iter = 50; %Number of iterations
R = pi; %Radius of convergence for classic Mandlebrot set 'trajectories'
DPI = 300; %DPI of output PNG file
fsize = 16; %Fontsize for plots

%Display message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Classic Mandelbrot Plotter ' )
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Start clock
t0 = tic;

%Define Argand diagram
x = linspace( -2,1,2*N); y = linspace( -1,1,N);
[x,y] = meshgrid(x,y);
z0 = x + 1i*y; z = z0;

%Perform Mandlebrot iteration
M = NaN(size(z));
passed_test = zeros(size(z));
for n=1:iter
    
    %Apply mandlebrot transformation to entire Aragand diagram
    z = z.^2 + z0;
    
    %Check whether any elements of z pass the Mandlebrot test for this
    %iteration (don't include passes for previous iterations)
    pass = (passed_test==0) & ( abs(z) > R );
    passed_test( pass ) = 1;
    M(pass) = n;
    disp([' Iteration ',num2str(n),' of ',num2str(iter),' ...'])    
end

%Plot argand surface
disp(' ')
disp(' Generating surface plots, please be patient ')
figure('name','mandelbrot surface','color',[1 1 1]);
surf( x,y,exp(-abs(z))); view(2);
colormap prism; shading interp; interp_colormap(1000);
axis tight; axis equal; axis vis3d; box on; grid off;
set(gca,'fontsize',fsize); 
xlabel('x'); ylabel('y'); zlabel('exp(-abs(z))'); title('Mandlebrot surface')
print( gcf, 'mandlebrot surface.png',['-r',num2str(DPI)],'-dpng' );

%Plot classic mandelbrot
figure('name','mandelbrot iteration','color',[1 1 1]);
surf( x,y,M); view(2)
caxis([1,iter]); colormap prism
shading interp; interp_colormap(1000); 
axis tight; axis equal; box on; grid off
set(gca,'fontsize',fsize);
xlabel('x'); ylabel('y'); zlabel('Mandlebrot iteration')
title('Mandlebrot iteration');
print( gcf, 'mandlebrot classic.png',['-r',num2str(DPI)],'-dpng' );
disp(' ')
disp([' Classic Mandelbrot plots created in ',num2str(toc(t0)),' seconds.'])
disp(' ')

%%

% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1)); G = ones(1,dim(1)); B = ones(1,dim(1));
RR = ones(1,N); GG = ones(1,N); BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1); G(:) = map(:,2); B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:); new_map(:,2) = GG(:); new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code
