%Allows for exploration of the Mandlebrot set
function mandelexplorer
global d
%Mandlebrot grid size and Aragand diagram limits
d.N = 150; d.xc = -0.5; d.xw = 3; d.yc = 0; d.yw = 2;
d.iter = 123;  %Mandlebrot iterations
d.R = pi; %Radius of convergence
d.fsize = 18;  %Fontsize for graphs
d.NaN_RGB = [1,1,1];

%

%Make initial Mandelbrot colour matrix
[d.x,d.y,d.M] = make_mandelbrot( d.xc,d.xw,d.yc,d.yw,d.N,d.R,d.iter );

%Plot classic Mandelbrot
d.fig = figure('keypressfcn',@keypress,'WindowButtonUpFcn',@mouseclickup,...
    'units','normalized','position',[0 0 1 1],...
    'numbertitle','off','name','Mandelbrot Explorer - by Andy French 2020');
mandelbrot_plot;

%%

%Mandelbrot plot
function mandelbrot_plot
global d
figure(d.fig); clf; cmap = colormap('prism'); interp_colormap(1000);
caxis([1,d.iter]);  set(gca,'fontsize',d.fsize);

if d.xw>0.01
    d.s = pcolor( d.x,d.y,d.M); view(2); shading flat;
else
    C=make_colour_matrix( d.M,cmap,d.NaN_RGB,[1,d.iter] );
    x = linspace( d.xc - d.xw/2,d.xc + d.xw/2,2*d.N);
    y = linspace( d.yc - d.yw/2,d.yc + d.yw/2,d.N);
    d.s = image(x,y,uint8( 255*C) );
end

axis equal; axis tight; box on; grid off;
xlabel('x'); ylabel('y'); box on; grid off
d.tit = title( ['x=',num2str(d.xc),' +/- ',num2str(d.xw),...
    ', y=',num2str(d.yc),' +/- ',num2str(d.yw),...
    ', N=',num2str(d.N),', I=',num2str(d.iter)] );

%%

%Construct Mandlebrot matrix M
function [x,y,M] = make_mandelbrot( xc,xw,yc,yw,N,R,I )

%Define argand diagram
x = linspace( xc - xw/2,xc + xw/2,2*N);
y = linspace( yc - yw/2,yc + yw/2,N);
[x,y] = meshgrid(x,y); z = x + 1i*y;

%Perform Mandlebrot iteration. The indexed elements of z are ind.
dim = size(z); z = reshape( z, [1,numel(z)] );  z0 = z;
M = NaN(size(z)); ind = 1:numel(M);
for n=1:I
    
    %Apply mandlebrot transformation to Aragand diagram elements
    %that haven't yet passed the test. This is a nifty speed boost!
    z = z.^2 + z0;
    
    %Check whether any (remaining) elements of z pass the Mandlebrot test
    pass = ( ( abs(real(z)) > R ) & ( abs(imag(z)) > R ) );
    
    %Store iteration number for elements of Argand diagram that
    M( ind(pass) ) = n;
    
    %Remove passes from array index vector
    z(pass) = []; z0(pass) = []; ind(pass) = [];
end
M = reshape( M,dim );

%%

% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap( N )

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1)); G = ones(1,dim(1)); B = ones(1,dim(1));
RR = ones(1,N); GG = ones(1,N); BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1); G(:) = map(:,2); B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:); new_map(:,2) = GG(:); new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%%

% Function that takes a two dimensional array of intensities
% and converts this into an RGB array for use in the imwrite.m function
% A          - M x N array of 'intensities', these should be normalised such as
%              the magnitudes will be in the interval [0,1.
% cmap       - Colour map. i.e.  cmap=colormap('prism')
% NaN_colour - [R,G,B] vector of NaN colours
% C          - M x N x 3 RGB matrix. Each page corresponds to RGB colurs
%             (respectively) in the range [0,1];
% clim       - Vector which maps normalized julia map to colormap range
function C=make_colour_matrix(A,cmap,NaN_colour,clim)

%Get dimensions of colourmap and initialise red, green and blue vectors
dim=size(cmap); N = dim(1);
red=zeros(1,N); green=zeros(1,N); blue=zeros(1,N);
red(:)=cmap(:,1); green(:)=cmap(:,2); blue(:)=cmap(:,3);

%Define colour scale clim to enable the colormap to be interpolated
cscale=linspace( clim(1),clim(2),N );

%Initialise output RGB matrix
C = zeros( [size(A),3] );

%Determine R,G,B components of C using linear interpolation of the
%colormap. Set NaNs to be the specified NaN colour.
R=interp1( cscale,red,A,'linear' );
R( isnan(R)==1 ) = NaN_colour(1);
G=interp1( cscale,green,A,'linear' );
G( isnan(G)==1 ) = NaN_colour(2);
B=interp1( cscale,blue,A,'linear' );
B( isnan(B)==1 )= NaN_colour(3);
C(:,:,1)=R; C(:,:,2)=G; C(:,:,3)=B;

%%

%Key press handler
function keypress(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( gcf,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit GUI
    close(gcf);
    
elseif strcmp(s,'uparrow') == 1
    %Increase grid size
    d.N = d.N+100;
    set( d.tit, 'string',['x=',num2str(d.xc),' +/- ',num2str(d.xw),...
        ', y=',num2str(d.yc),' +/- ',num2str(d.yw),...
        ', N=',num2str(d.N),', I=',num2str(d.iter)] );
    
elseif ( strcmp(s,'downarrow') == 1 )
    %Decrease grid size
    d.N = d.N-100;
    if d.N<100; d.N = 100; end
    set( d.tit, 'string',['x=',num2str(d.xc),' +/- ',num2str(d.xw),...
        ', y=',num2str(d.yc),' +/- ',num2str(d.yw),...
        ', N=',num2str(d.N),', I=',num2str(d.iter)] );
    
elseif strcmp(s,'leftarrow') == 1
    %Reduce iterations
    d.iter = d.iter-1;  if d.iter<1; d.iter = 1; end
    set( d.tit, 'string',['x=',num2str(d.xc),' +/- ',num2str(d.xw),...
        ', y=',num2str(d.yc),' +/- ',num2str(d.yw),...
        ', N=',num2str(d.N),', I=',num2str(d.iter)] );
    
elseif ( strcmp(s,'rightarrow') == 1 )
    %Increase iterations
    d.iter = d.iter+1;
    set( d.tit, 'string',['x=',num2str(d.xc),' +/- ',num2str(d.xw),...
        ', y=',num2str(d.yc),' +/- ',num2str(d.yw),...
        ', N=',num2str(d.N),', I=',num2str(d.iter)] );
    
elseif ( strcmp(s,'p') == 1 )
    %Print screen to a PNG file
    filename = [ strrep( ['x=',num2str(d.xc),'+-',num2str(d.xw),...
        ' y=',num2str(d.yc),'+-',num2str(d.yw),...
        ' N=',num2str(d.N),' I=',num2str(d.iter)],'.','p'),'.png' ];
    set( gca,'fontsize',10 ); xlabel('x','fontsize',10);
    set( d.tit,'fontsize',10 ); ylabel('y','fontsize',10);
    print( gcf, filename,'-dpng','-r300');
    set( gca,'fontsize',d.fsize ); xlabel('x','fontsize',d.fsize);
    set( d.tit,'fontsize',d.fsize ); ylabel('y','fontsize',d.fsize);
    
elseif ( strcmp(s,'u') == 1 )
    
    %Zoom out
    F = 1.5; d.xw = F*d.xw; d.yw = F*d.yw;
    
    %Update Mandelbrot plot
    [d.x,d.y,d.M] = make_mandelbrot( d.xc,d.xw,d.yc,d.yw,d.N,d.R,d.iter );
    mandelbrot_plot;
    
    %Display code for mandleposter to command window
    disp(['xc = ',num2str(d.xc,15),'; yc = ',num2str(d.yc,15),...
        '; xw = ',num2str(d.xw,15),'; iter = ',num2str(d.iter),';']);
    
elseif ( strcmp(s,'z') == 1 )
    
    %Zoom in
    F = 1.5; d.xw = d.xw/F; d.yw = d.yw/F;
    
    %Update Mandelbrot plot
    [d.x,d.y,d.M] = make_mandelbrot( d.xc,d.xw,d.yc,d.yw,d.N,d.R,d.iter );
    mandelbrot_plot;
    
    %Display code for mandleposter to command window
    disp(['xc = ',num2str(d.xc,15),'; yc = ',num2str(d.yc,15),...
        '; xw = ',num2str(d.xw,15),'; iter = ',num2str(d.iter),';']);
    
else
    
    %Regenerate screen
    
    %Update axis limits
    xlimits = get( gca,'xlim'); ylimits = get( gca,'ylim');
    d.xc = mean(xlimits); d.xw = diff(xlimits);
    d.yc = mean(ylimits); d.yw = diff(ylimits);
    
    %Update Mandelbrot plot
    [d.x,d.y,d.M] = make_mandelbrot( d.xc,d.xw,d.yc,d.yw,d.N,d.R,d.iter );
    mandelbrot_plot;
    
    %Display code for mandleposter to command window
    disp(['xc = ',num2str(d.xc,15),'; yc = ',num2str(d.yc,15),...
        '; xw = ',num2str(d.xw,15),'; iter = ',num2str(d.iter),';']);
end

%%

%Mouse click up function
function mouseclickup(hObject, eventdata, handles)
global d

%Regenerate screen

%Update axis limits
xlimits = get( gca,'xlim'); ylimits = get( gca,'ylim');
d.xc = mean(xlimits); d.xw = diff(xlimits);
d.yc = mean(ylimits); d.yw = diff(ylimits);

%Update Mandelbrot plot
[d.x,d.y,d.M] = make_mandelbrot( d.xc,d.xw,d.yc,d.yw,d.N,d.R,d.iter );
mandelbrot_plot;

%Display code for mandleposter to command window
disp(['xc = ',num2str(d.xc,15),'; yc = ',num2str(d.yc,15),...
    '; xw = ',num2str(d.xw,15),'; iter = ',num2str(d.iter),';']);

%End of code