%Animates an exploration of the Mandelbrot
% complex plane transformation and produces an AVI file.
function mandelmovie
N = 900;      %Animation frames
H = 1080;       %Pixel height of output
W = 1920;       %Pixel width of (square) tile
iter = 200;     %Mandlebrot iterations
R = pi;         %Radius of convergence

%Colour for areas within Mandelbrot set
NaN_RGB = [1,1,1];

%Centre of image and initial image width
xc = -1.71138881126865; yc = -6.50469771463967e-007; xw = 10;

%Determine vector of zoom factors
zf = logspace(0,11,N);

%

%Start clock
t0 = tic; disp(' '); disp(' Generating Mandelbrot movie ...');

%Get colormap
figure; cmap = colormap('prism'); close(gcf);

%Create movie object
V = VideoWriter( ['mandelmovie ',strrep( datestr(now),':','-' ),'.avi']);
V.FrameRate = 30; open(V);
disp(' '); disp(' Generating video frames ... '); d.frame = 0;

%Create movie frames ....
for j=1:N
    %Make mandlebrot matrix M
    [x,y,M] = make_mandelbrot( xc,yc,xw/zf(j),W,H,R,iter,j );
    if numel( find( isnan(M)==1 ) ) == numel(M)
        %All elements are NaN, so further zoom won't change this!
        break
    end
    
    %Convert to an RGB colour matrix
    C=make_colour_matrix( M,cmap,NaN_RGB,[1,iter] );
    
    %Add frame to video
    writeVideo( V,uint8(C*255) );
    disp([' Frame ',leadzero(4,d.frame),' added to video ...'] );
    d.frame = d.frame + 1;
end

%Close AVI file and display completion message
close(V);
disp([' Mandelmovie generated in ',num2str(toc(t0)/60),' minutes.']);
disp(' ');

%%

% Function that takes a two dimensional array of intensities
% and converts this into an RGB array for use in the imwrite.m function
% A          - M x N array of 'intensities', these should be normalised such as
%              the magnitudes will be in the interval [0,1.
% cmap       - Colour map. i.e.  cmap=colormap('prism')
% NaN_colour - [R,G,B] vector of NaN colours
% C          - M x N x 3 RGB matrix. Each page corresponds to RGB colurs
%             (respectively) in the range [0,1];
% clim       - Vector which maps normalized julia map to colormap range
function C=make_colour_matrix(A,cmap,NaN_colour,clim)

%Get dimensions of colourmap and initialise red, green and blue vectors
dim=size(cmap); N = dim(1);
red=zeros(1,N); green=zeros(1,N); blue=zeros(1,N);
red(:)=cmap(:,1); green(:)=cmap(:,2); blue(:)=cmap(:,3);

%Define colour scale clim to enable the colormap to be interpolated
cscale=linspace( clim(1),clim(2),N );

%Initialise output RGB matrix
C = zeros( [size(A),3] );

%Determine R,G,B components of C using linear interpolation of the
%colormap. Set NaNs to be the specified NaN colour.
R=interp1( cscale,red,A,'linear' );
R( isnan(R)==1 ) = NaN_colour(1);
G=interp1( cscale,green,A,'linear' );
G( isnan(G)==1 ) = NaN_colour(2);
B=interp1( cscale,blue,A,'linear' );
B( isnan(B)==1 )= NaN_colour(3);
C(:,:,1)=R; C(:,:,2)=G; C(:,:,3)=B;

%%

%Construct Mandlebrot matrix M
function [x,y,M] = make_mandelbrot( xc,yc,xw,W,H,R,I,j )

%Define argand diagram
yw = xw*H/W;
x = linspace( xc-xw/2,xc+xw/2,W );
y = linspace( yc-yw/2,yc+yw/2,H );
[x,y] = meshgrid(x,y); z = x + 1i*y;

%Perform Mandlebrot iteration. The indexed elements of z are ind.
dim = size(z); z = reshape( z, [1,numel(z)] );  z0 = z;
M = NaN(size(z)); ind = 1:numel(M);
for n=1:I    
    %Apply mandlebrot transformation to Aragand diagram elements
    %that haven't yet passed the test. This is a nifty speed boost!
    z = z.^2 + z0;
    
    %Check whether any (remaining) elements of z pass the Mandlebrot test
    pass = ( ( abs(real(z)) > R ) & ( abs(imag(z)) > R ) );
    
    %Store iteration number for elements of Argand diagram that
    M( ind(pass) ) = n;
    
    %Remove passes from array index vector
    z(pass) = []; z0(pass) = []; ind(pass) = [];
end
M = reshape( M,dim );

%%

%Leading zero function
function s=leadzero(Nz,z)
fmt=num2str(Nz,'%%0%dg'); s=sprintf(fmt,z);

%End of code