%This function generates a 2D mandlebrot image in a series of bitmap
%tiles, which are then combined to make a high resolution image.
function mandelposter
H = 4000;       %Pixel height of output
w = 1000;       %Pixel width of (square) tile
a = 2/3;        %Aspect ratio heigh/width
iter = 123;     %Mandlebrot iterations
R = pi;         %Radius of convergence

%Colour for areas within Mandelbrot set
NaN_RGB = [1,1,1];

%Default entre of image and image width
xc = -0.5; yc = 0;  xw = 3;

%Paste from mandelexplorer output
xc = -1.7110489389361; yc = -4.90334272257057e-005; xw = 0.00222648827813732; iter = 123;

%

%Start clock
t0 = tic; disp(' '); disp(' Generating Mandelbrot poster ...');

%Get colormap
figure; cmap = colormap('prism'); close(gcf);

%Calculate output image width output in pixels
W = round(H/a);

%Create full size output colour matrix
C = zeros( H,W,3);

%Create full Argand diagram
h = xw*a/2;
x = linspace( xc-xw/2,xc+xw/2,W ); 
y = linspace( -h+yc,h+yc, H);

%Determine tile sizes in pixels
num_x = ceil(W/w); num_y = ceil( H/w ); 
rem_x = mod( W, w ); rem_y = mod( H, w );
num_tiles = num_x*num_y;

%Step through Argand diagram and determine elements of colour matrix C
num = 0;
for nx = 1:num_x
    for ny = 1:num_y
        
        %Increment tile number
        num = num + 1;
        disp(' '); disp([' Tile ',num2str(num),' of ',num2str(num_tiles),' ...'])
        
        %Define indices of section of the Argand diagram to be processed
        if ((nx < num_x) || (rem_x == 0))
            indx = ((nx-1)*w + 1):(nx*w);
        else
            indx = ((nx-1)*w + 1):((nx-1)*w + rem_x);
        end       
        if ((ny < num_y) || (rem_y == 0))
            indy = ((ny-1)*w + 1):(ny*w);
        else
            indy = ((ny-1)*w + 1):((ny-1)*w+ rem_y);
        end
        
        %Define subset of initially defined Argand diagram
        xn = x(indx); yn = y(indy); [xn,yn]=meshgrid(xn,yn);
        z = xn + 1i*yn;
        
        %Perform Mandlebrot iteration. The indexed elements of z are ind.
        dim = size(z); z = reshape( z, [1,numel(z)] );  z0 = z;
        M = NaN(size(z)); ind = 1:numel(M);
        for n=1:iter
            
            %Apply mandlebrot transformation to Aragand diagram elements
            %that haven't yet passed the test. This is a nifty speed boost!
            z = z.^2 + z0;
            
            %Check whether any (remaining) elements of z pass the Mandlebrot test 
            pass = ( ( abs(real(z)) > R ) & ( abs(imag(z)) > R ) );
            
            %Store iteration number for elements of Argand diagram that
            M( ind(pass) ) = n;
            
            %Remove passes from array index vector
            z(pass) = []; z0(pass) = []; ind(pass) = []; 
            %disp([' Iteration ',num2str(n),' of ',num2str(iter),' ...'])
        end
        M = reshape( M,dim ); 
        
        %Add to colour matrix
        C(indy,indx,:) = make_colour_matrix( M,cmap,NaN_RGB,[0,iter]);
    end
end
clear M x y z xn yn ind indx indy pass z0

%Write out bitmap image
C = uint8(255*flipdim(C,1));
disp([' Writing PNG image ...']); imwrite(C,'julia.png','png');
disp([' Julia bitmap generated in ',num2str(toc(t0)/60),' minutes.']);
disp(' ');

%%

% Function that takes a two dimensional array of intensities
% and converts this into an RGB array for use in the imwrite.m function
% A          - M x N array of 'intensities', these should be normalised such as
%              the magnitudes will be in the interval [0,1.
% cmap       - Colour map. i.e.  cmap=colormap('prism')
% NaN_colour - [R,G,B] vector of NaN colours
% C          - M x N x 3 RGB matrix. Each page corresponds to RGB colurs
%             (respectively) in the range [0,1];
% clim       - Vector which maps normalized julia map to colormap range
function C=make_colour_matrix(A,cmap,NaN_colour,clim)

%Get dimensions of colourmap and initialise red, green and blue vectors
dim=size(cmap); N = dim(1);
red=zeros(1,N); green=zeros(1,N); blue=zeros(1,N);
red(:)=cmap(:,1); green(:)=cmap(:,2); blue(:)=cmap(:,3);

%Define colour scale clim to enable the colormap to be interpolated
cscale=linspace( clim(1),clim(2),N );

%Initialise output RGB matrix
C = zeros( [size(A),3] );

%Determine R,G,B components of C using linear interpolation of the
%colormap. Set NaNs to be the specified NaN colour.
R=interp1( cscale,red,A,'linear' );
R( isnan(R)==1 ) = NaN_colour(1);
G=interp1( cscale,green,A,'linear' );
G( isnan(G)==1 ) = NaN_colour(2);
B=interp1( cscale,blue,A,'linear' );
B( isnan(B)==1 )= NaN_colour(3);
C(:,:,1)=R; C(:,:,2)=G; C(:,:,3)=B;

%End of code